/*
 * FILE --- fms.cc
 *
 * PURPOSE --- Source file for a class that provides an example of usage
 * of the Limitied Lookeahead Policy (LLP) controller. It simulates a load 
 * blanacing problem in a six machine Flexible Manufacturing System (FMS).
 *
 */

// Include Files
#include <math.h>
#include <stdio.h>
#include "fms.hh"

FMS::FMS(int _plant_order, int _ctrl_order, 
	 int _lookahead_horizon, int _num_diff_ctrl)
  : LLP(_plant_order, _ctrl_order, _lookahead_horizon, _num_diff_ctrl)
{
  // Allocate memory for the balances state
  if ((balanced_state = new double[plant_order]) == NULL)
    printf("Memory error!!!!\n");
}

FMS::~FMS()
{
  // Delete the allocated memory
  delete [] balanced_state;
}


double FMS::find_cost(const double *_state, const double *_control)
{
  double J = 0.0;

  // Find the cost for the given state and control
  for(int ii = 0; ii < plant_order; ii++)
    {
      J = J + fabs(_state[ii] - balanced_state[ii]);
    }
  
  return (J);
}


void FMS::dynamics(const double *_state, const double *_control, double *_next_state)
{
  // Implement the system dynamics for the FMS
  // in the form of x(k+1) = F(x(k),u(k))

  // For the FMS we have  x(k+1) = x(k) + u(k)
  for(int ii = 0; ii < plant_order; ii++)
    {
      _next_state[ii] = _state[ii] + _control[ii];
    }
}

void FMS::set_state(const double *_initial_state)
{
  // Set the system state to the specified value 
  for(int ii = 0; ii < plant_order; ii++)
    {
      plant_state[ii] = _initial_state[ii];
    }

  // Given the new state calculate the corresponding balanced state
  find_balanced_state(); 
}

void FMS::find_balanced_state(void)
{
  int ii; // Index variales for the loops
  double tot_parts = 0.0;
  double avrg_parts;
  int rm_parts;

  // Calculate the total number of parts in the system
  for(ii = 0; ii < plant_order; ii++)
    {
      tot_parts = tot_parts + plant_state[ii];
    }
  
  // Calculate the average number of parts
  avrg_parts = floor((double) tot_parts/plant_order);
  
  // Assign the balanced state (core part)
  for(ii = 0; ii < plant_order; ii++)
    {
      balanced_state[ii] = avrg_parts;
    }
  
  // If not equally balanced then find the remeining parts
  rm_parts = (int) (1*(tot_parts - plant_order*avrg_parts));

  // Distribute the extra parts among the first machines
  for(ii = 0; ii < rm_parts; ii++)
    {
      balanced_state[ii] = balanced_state[ii] + 1.0;
    }
}
